/*
 * FILE: driver.c
 * AUTH: Soren Spies (sspies)
 * DATE: 5 Feb 2001
 * DESC: test driver for STunnel project
 *
 * Copyright (c) 2001 Soren Spies.  All rights reserved except as noted
 * in the included included file COPYING.
 * STunnel is available from http://www.techhouse.org/~soren/software
 */

/*#define DEBUG*/	/* change me to enable debug output in this file*/

#include "STunnel.h"
#include "util.h"
#include <stdio.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pwd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>

int main(int argc, char **argv) {
	STReturn_t state;
	struct sockaddr_in src, dst;
	STOpts opts;
	STConRef con;
	struct hostent *hent;

	/* change (and uncomment) this section to test the STDeleteKey function */
	/* 
	if(STDeleteKey("netspace.org"))
		printf("STDeleteKey(\"netspace.org\") failed\n");
	else
		printf("STDeleteKey(\"netspace.org\") succeeded\n");
	*/

	/* this opens a tunnel from localhost to the remote host ... meaning that
	 * the listening socket is on the local machine ... putting 
	 * INADDR_LOOPBACK into dst would make the tunnel point the other way.
	 * Note that src is the end of the tunnel that is listening, not
	 * necessarily the local or remote host (likewise dst is not always
	 * the host you are connecting to. */

	printf("setting up for ssh -L3333:localhost:4444 soren@techhouse.org\n");
	hent = gethostbyname("techhouse.org");	/* change me to change host */
	if(!hent) {
		herror("gethostbyname unhappy");
		exit(1);
	}

	memcpy((void*)&dst.sin_addr.s_addr, (void*)hent->h_addr, hent->h_length);
	dprintf(("inet: %s\n", inet_ntoa(dst.sin_addr)));
	dst.sin_port = 4444;
	src.sin_addr.s_addr = INADDR_LOOPBACK;
	src.sin_port = 3333;
	opts.login = "soren";	/* change me to change account on remote host */

	state = STOpenTunnel(&src, &dst, opts, &con);

	if(STErr <= state)
		errexit("STOpenTunnel failed");
	while(state != STSuccess) {
		STContParams cparams;
		char resp[_PASSWORD_LEN];

		switch(state) {
			case STNewHostKey:
				printf("okay to accept new host key (y/n)? ");
				fflush(stdout);
				fgets(resp, _PASSWORD_LEN, stdin);
				if(*resp == 'y' || *resp == 'Y') {
					cparams.goahead = 1;
					state = STContOpen(con, &cparams);
				}
				else {
					printf("closing tunnel b/c we don't like the new key\n");
					state = STCloseTunnel(con);
					if(state == STSuccess)
						state = STErr;
					exit(1);
				}

				break;

			case STPasswordNeeded:
				cparams.passphrase = getpass("give me a password for ssh: ");
				state = STContOpen(con, &cparams);

				break;

			case STPassphraseNeeded:
				cparams.passphrase = getpass("a passphrase for your key: ");
				state = STContOpen(con, &cparams);

				break;

			case STHostKeyChanged:
				printf("deleting key for %s and retrying "
						"(your prog should ask about this)\n",
						inet_ntoa(dst.sin_addr));
				if((state = STDeleteKey(inet_ntoa(dst.sin_addr))) == STSuccess)
					state = STOpenTunnel(&src, &dst, opts, &con);

				break;

			case STPermissionDenied:
				errexit("access denied\n");
				break;

			default:
				printf("exit'ing with error state: %d\n", state);
				exit(1);
		}
	}

	/* redundant since above has been changed to exit in all error conditions */
	if(state != STSuccess) {
		dprintf(("failure state: %d\n", state));
		errexit("connection did not open");
	}

	dprintf(("tunnel should be open?\n"));
	printf("hit return after testing the tunnel: ");
	fflush(stdout);
	fgetc(stdin);
	//scanf("\n");
	STCloseTunnel(con);
	exit(0);
}
