/*
 * FILE: util.h
 * NAME: Soren Spies (scs)
 * DATE: 12 March 2001
 *
 * These are little utility macros that I want in more than one file.  I 
 * felt dirty putting them in the same file ... and I figure that this file
 * will grow to be more useful for me as a systems programmer over time.
 *
 * This file is Copyright (c) Soren Spies 2001.
 * Please see included file COPYING for license details.
 * STunnel is available from http://www.techhouse.org/~soren/software
 */

#ifndef UTIL_H
#define UTIL_H

#include <sys/types.h>


#define perrexit(str) do { \
	perror(str); \
	exit(1); \
	} while(0)

#define errexit(str) do { \
	fprintf(stderr, "%s\n", str); \
	exit(1); \
	} while(0)

#define errprint(str) fprintf(stderr, str)

#ifndef dprintf
#ifdef DEBUG
#define dprintf(args) do { printf args; fflush(stdout); } while(0)
#define dperror(msg) do { perror(msg); } while(0)
#else
#define dprintf(args) do { } while(0)
#define dperror(msg) do { } while(0)
#endif	/* DEBUG */
#endif	/* dprintf */


#endif	/* ndef UTIL_H */
